#ifndef _BADPENGUIN_HARDWARE_H
#define _BADPENGUIN_HARDWARE_H

/*  libhardware -- generic library for detecting Hardware
 *  $Id: badpenguin-hardware.h,v 1.6 2002/10/09 13:34:15 agx Exp $
 *
 *  Copyright (C) 1996 Antonio Gallo <agx@linux.it>
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */


// includes
#include <badpenguin-chain.h>


/* typedefs */
typedef struct stComputer    TComputer;
typedef struct stPciDevice   TPciDevice;
typedef struct stBlockDevice TBlockDevice;
typedef struct stPartition   TPartition;
typedef struct stNetDevice   TNetDevice;

/* macro for typecasting */
#define TCOMPUTER(X)    ((TComputer*)(X))
#define TPCIDEVICE(X)   ((TPciDevice*)(X))
#define TBLOCKDEVICE(X) ((TBlockDevice*)(X))
#define TPARTITION(X)   ((TPartition*)(X))
#define TNETDEVICE(X)	((TNetDevice*)(X))

/* structures */
struct stComputer {
  TChain *pci;
  TChain *eide;
  TChain *scsi;
  TChain *partitions;
};


#define MODULE_STATUS_NOT_FOUND		0
#define MODULE_STATUS_UNSUPPORTED	1
#define MODULE_STATUS_GENERIC		10
#define MODULE_STATUS_DETECTED		11
#define MODULE_STATUS_RUNNING		20

/* PCI data structure TPciDevice */
struct stPciDevice {
  unsigned int  vendor;
  unsigned int  device;
  unsigned char class;
  unsigned char subclass;
  char *className;
  unsigned int  status;
      // 0 = driver not detected
      // 1 = generic module detected
      // 2 = module detected
      // 3 = module not found
  char *vendorName;
  char *deviceName;
  char *kernelModule;
};

/* Eide cdrom,disks & SCSI structure */
struct stBlockDevice {
  char *device;
  char *media;
  char *model;
};

struct stNetDevice {
  char *device;
  char *media;
  unsigned long RecvBytes;
  unsigned long SentBytes;
};

/* 
 * Media Types
 * Kernel enums for EIDE: disk, cdrom, tape, floppy, UNKNOWN
 * Kernel enums for SCSI: Direct-Access, Sequential-Access, Printer, 
 *                        Processor, WORM. CD-ROM, Scanner, Optical Device
 *                        Medium Changer, Communications, Unknown, Enclosure
 */
#define MEDIA_UNKNOWN		"unknown"
#define MEDIA_DISK_EIDE		"disk"
#define MEDIA_CDROM_EIDE	"cdrom"
#define MEDIA_DISK_SCSI		"Direct-Access"
#define MEDIA_CDROM_SCSI	"CD-ROM"
#define MEDIA_TAPE_SCSI		"Sequential-Access"
#define MEDIA_ETH		"Ethernet"
#define MEDIA_LO		"Loopback"
#define MEDIA_PPP		"Point-to-Point"

// Model description
#define MODEL_UNKNOWN		"Device sconosciuto"

/* Partitions attached to a device */
struct stPartition {
  char *device;
  unsigned int  type;
  char *filesystem;
  char *module;
  unsigned int  bootflag;
  unsigned long size;
};

// Filesytems
#define FILESYSTEM_ISO9660	"iso9660"
#define FILESYSTEM_EXT2		"ext2"
#define FILESYSTEM_REISER  	"reiserfs"
#define FILESYSTEM_SWAP		"swap"
#define FILESYSTEM_MSDOS	"msdos"
#define FILESYSTEM_VFAT		"vfat"
#define FILESYSTEM_NONE		"none"


/*  F U N C T I O N S  */
int libhardware_version();

TComputer  *create_computer();
TPartition *create_partition();

TChain *query_pci_devices (TChain *devices, int loiVerbose);
int parse_pci_devices(TChain *pcichain);
int check_pci_devices(TChain *pcichain);

TChain *query_eide_devices(TChain *devices);
TChain *query_scsi_devices(TChain *devices);	
TChain *query_partitions(TChain *part, TChain *eide, TChain *scsi);
TChain *query_net_devices(TChain *devices);

int load_pci_device(TPciDevice *dev);

// used to change the default directories at runtime
int set_proc_dir(char *pdir, char *ddir);
int set_new_dir(char *pdir, char *ddir);


/* =================== PCI classes and subclasses ================= */
#define PCI_CLASS_NOT_DEFINED	0x00
#define PCI_CLASS_STORAGE	0x01
#define PCI_CLASS_STORAGE_SCSI	0x00

#endif
