#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "badpenguin-hardware.h"
#include "set-proc-dir.h"


int load_pci_device(TPciDevice *dev)
{
	int i;
	pid_t cpid;
	int status;
	sigset_t nmask, omask;

	// avoid non-supported modules
	if ( dev->status != MODULE_STATUS_GENERIC  && dev->status != MODULE_STATUS_DETECTED ) {
		errno = EINVAL;
		return(-1);
	}
	if ( dev->kernelModule == NULL ) {
		errno = EINVAL;
		return(-1);
	}
	// DISPLAY should not loaded via modprobe, it's an X-Server :-)
	if ( 0 == strcmp( dev->className, "DISPLAY" ) ) {
		errno = EINVAL;
		return(-1);
	}


	if(geteuid() != 0) {
		errno = EPERM;
		fprintf(stderr, "libgringo: only the root user can load kernel modules\n");
		return(-1);
	}

	if((cpid = fork()) < 0) {
		fprintf(stderr, "libgringo: fork failed\n");
		return(-1);
	}

	if(cpid == 0) {
		char *args[5];
		struct sigaction sig;

		sig.sa_handler = SIG_IGN;
		sig.sa_flags = 0;
		sigemptyset(&sig.sa_mask);

		for(i = 0; i < NSIG; i++)
			sigaction(i, &sig, NULL);

// THIS WILL AVOID ANYTHING TO BE DISPLAYED 
//		for(i = 0; i < FD_SETSIZE; i++)
//			close(i);

		args[0] = DEFAULT_MODPROBE_PATH;
		args[1] = "-s";
		args[2] = "-k";
		args[3] = dev->kernelModule;
		args[4] = NULL;
		fprintf(stderr, "libgringo: loading module '%s'\n", args[3] );
		execv(args[0], args);
		_exit(127);
	}

	sigfillset(&nmask);
	sigdelset(&nmask, SIGKILL);
	sigdelset(&nmask, SIGSTOP);
	sigprocmask(SIG_SETMASK, &nmask, &omask);

	while(waitpid(cpid, &status, 0) != cpid)
		;

	sigprocmask(SIG_SETMASK, &omask, NULL);

	if(WIFEXITED(status)) {
       		if(WEXITSTATUS(status) == 127) {
			errno = ENOEXEC;
			return(-1);
		}
		else if(WEXITSTATUS(status) != 0) {
			/* this is really the wrong errno, but what
		 	* should I use instead? */
			errno = EINVAL;
			return(-1);
		}
	}
	else {
		/* this is really the wrong errno, but what
		* should I use instead? */
		errno = EINVAL;
		return(-1);
	}

	dev->status = MODULE_STATUS_RUNNING;

	return(0);
}
