#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <badpenguin-fs.h>
#include "badpenguin-hardware.h"
#include "hardware.h"
#include "set-proc-dir.h"
#include "hexutils.h"

/* Device Info
 * This source is used to parse a PCI device database in order to find:
 * - Vendor Name
 * - Device Name
 * - Kernel Module
 */



int find_char(char *s, char c) {
  int p=0;
  while (s[p]!=c) p++;
  if (s[p]==c) {
    return p; 
  } else {
    return -1;
  }
}



int parse_info_line(TPciDevice *infodevice, char *line) {
  char *templine = line;
  char *destline = NULL;
  int l=0;

  // skip vendor.device ID
  templine= templine + 10;
  // find module
  l = find_char(templine,'|');
  if (l>=0) {
    destline = (char*) calloc(l+1, sizeof(char));
    strncpy(destline, templine, l);
    infodevice->kernelModule = strdup(destline);
  } else
    return -1;
  // find vendor
  templine= templine + l + 1;
  l = find_char(templine,'|');
  if (l>=0) {
    destline = (char*) calloc(l+1, sizeof(char));
    strncpy(destline, templine, l);
    infodevice->vendorName = strdup(destline);
  } else
    return -1;
  // find device description
  templine= templine + l + 1;
  infodevice->deviceName = strdup(templine);
  return 0;
}



int check_device_info(TPciDevice *devicepci, char *detectfilename ) {
  FILE	*fd = NULL;
  char  line[256];
  int   infofound= 0;
  int   retcode = 0;
  char  detectfile[200];
  
  // Compose detect file name
  snprintf(detectfile, sizeof(detectfile), "%s/%s",
	   s_detect_dir, detectfilename);
  
  // open it
  fd = fopen(detectfile,"r");
  if (fd==NULL) {
    printf("Unable to open the detection list: %s!\n", detectfile);
    return -1;
  }
  while ( ! feof(fd) ) {
    read_line(fd, line);
    if ( ( getdword2(line) == devicepci->vendor ) &&
	 ( getdword2(line+5) == devicepci->device ) ) {
      infofound++;
      devicepci->status = MODULE_STATUS_DETECTED;
      break;
    }
    
  }
  if (infofound==0) {
    rewind(fd);
    while ( ! feof(fd) ) {
      read_line(fd, line);
      if ( ( getdword2(line) == devicepci->vendor ) &&
	   (0 == strncasecmp(line+5,"XXXX",4)) ) {
        infofound++;
	devicepci->status = MODULE_STATUS_GENERIC;
        retcode = 1;
        break;
      }    
    }    
  }
  fclose(fd);
  if (infofound==0) {
    devicepci->status = MODULE_STATUS_UNSUPPORTED;
    printf("No info found about the device %4x.%4x (%s)!\n",
	   devicepci->vendor, devicepci->device, devicepci->className);
    printf("  Please send to agx@linux.it the informations\n");
    printf("  about the PCI device you wanna put at work.\n");
    printf("  Thank you, Antonio Gallo.\n");
    sleep(5);
    return -1;
  }
  // Parse Detect Line
  parse_info_line(devicepci,line);
  return retcode;
}
