
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <badpenguin-fs.h>
#include "badpenguin-hardware.h"
#include "set-proc-dir.h"
#include "load-pci-device.h"


static int check_module_running(TPciDevice *device)
{
	FILE *fd;
	int c;
	char buf[256];
	char *bufPtr;
	int firstColumn = 1;

	if((fd = fopen("/proc/modules", "r")) == NULL) {
		fprintf(stderr, "libgringo: unable to open /proc/modules\n");
		return(-1);
	}
	bufPtr = buf;
	for(c = fgetc(fd); !feof(fd); c = fgetc(fd)) {
		if(c == '\n') {
			firstColumn = 1;
			bufPtr = buf;
			continue;
		}
		/* I only care about the first column of info produced */
		if(!firstColumn)
			continue;
		if(c == ' ' || c == '\t') {
			firstColumn = 0;
			*bufPtr = '\0';
			if(strcmp(buf, device->kernelModule) == 0) {
				fclose(fd);
				return(0);
			}
			continue;
		}
		/* ignore module names bigger then sizeof(buf) */
		if(bufPtr - buf >= sizeof(buf))
			firstColumn = 0;
		*bufPtr++ = c;
	}
	fclose(fd);

	return(-1);
}


int check_device_module(TPciDevice *devicepci ) {
  char tempstr[256];

  /* avoid unknwon modules or problems */
  if (devicepci->status < MODULE_STATUS_GENERIC ) return 0;
  if (devicepci->kernelModule == NULL ) return 0;

  /* check /proc/modules to see if module is already loaded */
  if(check_module_running(devicepci) == 0) {
	  devicepci->status = MODULE_STATUS_RUNNING;
	  return 0;
  }
  
  /* SCSI devices compiled statically into the kernel */
  if ( 0 == strcmp( devicepci->className, "SCSI") ) {
    snprintf(tempstr, sizeof(tempstr), "%s/%s/0",
	     s_proc_scsi, devicepci->kernelModule);
    if ( 0 == file_exists(tempstr) ) {
      devicepci->status = MODULE_STATUS_RUNNING;
      return 0;
    }
  }

  //DO NOT LOAD AUTOMATICALLY MODULES
  //so we return 0
  //return load_module(devicepci);
  return 0;
}



int check_pci_devices(TChain *pcichain) {
  TChainItem *pciitem = NULL;
  pciitem = pcichain->head;
  while (pciitem!=NULL) {
    if(check_device_module(pciitem->data) < 0)
      return -1;
    pciitem = pciitem->next;
  }
  return 0;
}

