#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "badpenguin-hardware.h"
#include "hardware.h"
#include "set-proc-dir.h"


int parse_pci_devices(TChain *pcichain) {
  TPciDevice *pcidata = NULL;
  TChainItem *pciitem = NULL;

  if ( initProc == 0 ) {
    set_proc_dir(DEFAULT_PROC_DIR, DEFAULT_DETECT_DIR );
  }

  pciitem = pcichain->head;
  while (pciitem!=NULL) {
    // get PCI data
    pcidata = pciitem->data;

    switch (pcidata->class) {
    
      /* UNDEFINED */
      case PCI_CLASS_NOT_DEFINED: {
        pcidata->className = strdup("NOT DEFINED");
	break;       
      }
      
      /* STORAGE */
      case PCI_CLASS_STORAGE: {
        switch (pcidata->subclass) {
	  /* SCSI */
	  case PCI_CLASS_STORAGE_SCSI: {
            pcidata->className = strdup("SCSI");
            check_device_info(pcidata, "scsi.db" );
	    break;
	  }
	  case 0x01: {
            pcidata->className = strdup("EIDE");
	    break;
	  }
	  case 0x02: {
            pcidata->className = strdup("FLOPPY");
	    break;
	  }
	  case 0x03: {
            pcidata->className = strdup("STORAGE - IPI");
	    break;
	  }
	  case 0x04: {
            pcidata->className = strdup("RAID");
            check_device_info(pcidata, "raid.db" );	    
	    break;
	  }
	  case 0x80: {
            pcidata->className = strdup("STORAGE - OTHER");
	    break;
	  }	  
        }
        break;
      }

      /* NETWORK */
      case 0x02: {
        switch (pcidata->subclass) {
	  case 0x00: {
            pcidata->className = strdup("ETHERNET");
            check_device_info(pcidata, "ethernet.db" );
	    break;
	  }
	  case 0x01: {
            pcidata->className = strdup("TOKENRING");
            check_device_info(pcidata, "ethernet.db" );
	    break;
	  }
	  case 0x02: {
            pcidata->className = strdup("FDDI");
            check_device_info(pcidata, "ethernet.db" );
	    break;
	  }
	  case 0x03: {
            pcidata->className = strdup("ATM");
            check_device_info(pcidata, "ethernet.db" );
	    break;
	  }
	  case 0x80: {
            pcidata->className = strdup("NETWORK - OTHER");
            check_device_info(pcidata, "ethernet.db" );
	    break;
	  }	  
        }
        break;
      }

      /* DISPLAY */
      case 0x03: {
        pcidata->className = strdup("DISPLAY");
        check_device_info(pcidata, "display.db" );
        break;
      }

      /* MULTIMEDIA */
      case 0x04: {
        switch (pcidata->subclass) {
	  case 0x00: {
            pcidata->className = strdup("VIDEO");
            check_device_info(pcidata, "display.db" );
	    break;
	  }
	  case 0x01: {
            pcidata->className = strdup("AUDIO");
            check_device_info(pcidata, "soundcard.db" );
	    break;
	  }
	  case 0x80: {
            pcidata->className = strdup("MULTIMEDIA - OTHER");
            check_device_info(pcidata, "soundcard.db" );
	    break;
	  }	  
        }
        break;
      }

      /* MEMORY */
      case 0x05: {
        pcidata->className = strdup("MEMORY");
        break;
      }

      /* BRIDGE */
      case 0x06: {
        pcidata->className = strdup("BRIDGE");
      
        switch (pcidata->subclass) {
	  case 0x05: {
	    free(pcidata->className);
            pcidata->className = strdup("PCMCIA");
            check_device_info(pcidata, "pcmcia.db" );
	    break;
	  }
	  case 0x07: {
	    free(pcidata->className);
            pcidata->className = strdup("CARDBUS");
            check_device_info(pcidata, "pcmcia.db" );
	    break;
	  }
        }

        break;
      }

      /* COMM PORT */
      case 0x07: {
        pcidata->className = strdup("COMM PORT");
	// 00=SERIAL 01=PARALLEL
        break;
      }

      /* SYSTEM */
      case 0x08: {
        pcidata->className = strdup("SYSTEM");
        break;
      }

      /* INPUT */
      case 0x09: {
        pcidata->className = strdup("INPUT");
	// 00=Keyboard 01=Pen 02=Mouse
        break;
      }

      /* DOCKING */
      case 0x0a: {
        pcidata->className = strdup("DOCKING");
        break;
      }

      /* PROCESSOR */
      case 0x0b: {
        pcidata->className = strdup("PROCESSOR");
        break;
      }

      /* SERIAL */
      case 0x0c: {
        pcidata->className = strdup("SERIAL");
//define PCI_CLASS_SERIAL_FIREWIRE	0x0c00
//define PCI_CLASS_SERIAL_ACCESS		0x0c01
//define PCI_CLASS_SERIAL_SSA		0x0c02
//define PCI_CLASS_SERIAL_FIBER		0x0c04
        switch (pcidata->subclass) {
	  case 0x03: {
	    free(pcidata->className);
            pcidata->className = strdup("USB");
            check_device_info(pcidata, "usb.db" );
	    break;
	  }
        }

        break;
      }

      /* HOT SWAP */
      case 0xff: {
        pcidata->className = strdup("HOT SWAP");
        break;
      }

    }

    // next
    pciitem = pciitem->next;
  }
  return 0;
}

