/*
 * ProbePart
 * This utility is part of libhardware
 * @2002 by Antonio Gallo <agx@linux.it> - www.badpenguin.org
 *
 * Options:
 *   -s   show size in sector (default)
 *   -b   show size in bytes
 *   -k   show size in kilo-bytes
 *   -m   show size in mega-bytes
 *   -g   show size in giga-bytes
 */
 
#include <stdio.h>
#include <getopt.h>
#include <badpenguin-chain.h>
#include "badpenguin-hardware.h"

int main(int argc, char* argv[]) {
	TChain *eide, *scsi, *part;
	TChainItem *c;
	TPartition *dev;
	char filter;
	long outsize;
	char o;
  
	filter = 's';
	while (( o = getopt(argc,argv,"bkmg")) != -1 ) {
		switch (o) {
			case 's':
			case 'b':
			case 'k': 
			case 'm': 
			case 'g': 
			{
				filter = o;
			}
		}
	}

	eide = query_eide_devices(NULL);
	scsi = query_scsi_devices(NULL);
	part = query_partitions(NULL,eide,scsi);

	c = part->head;
	while (c!=NULL) {
		dev = TPARTITION(c->data);
		outsize = dev->size / filter;
		switch (filter) {
			case 's': outsize=dev->size; break;
			case 'b': outsize=dev->size * 512; break;
			case 'k': outsize=dev->size / 2; break;
			case 'm': outsize=dev->size / 2048; break;
			case 'g': outsize=dev->size / (2048*1024); break;
			default : outsize=0;
		}
		printf( 
			"%s|%s|%lu|%-s\n", 
			dev->device, dev->module, outsize, dev->filesystem
		);
		c = c -> next;
	}

	return 0;
}
